@@ -1,29 +1,32 @@
+import warnings
+from logging import getLogger
+
 import matplotlib.pyplot as plt
 import pandas as pd
 import seaborn as sns
-from scipy.stats import chi2_contingency
 import statsmodels.api as sm
+from scipy.stats import chi2_contingency
 from statsmodels.formula.api import ols
 
+warnings.filterwarnings('ignore')
+
 
 class TTTH_Analyzer:
 
     def __init__(self):
-        pass
+        self.logger = getLogger(__class__.__name__)
 
-    @staticmethod
-    def analyze_category_variable(variable_name, df):
+    def analyze_category_variable(self, variable_name, df):
         """
         This function support to analyze category variable by count values and show bar plot of variable count
         parameter variable_name: name of categorical column
         parameter df: DataFrame include categorical column
         """
-        df_category = df[variable_name]
-        class_count = df_category.value_counts()
-        print(f'Class count of {variable_name}:\n')
-        print('==========')
-        print(f'{class_count}')
-        print('==========')
+        class_count = self._count_values_of_variable(variable_name, df)
+        self.logger.info(f'Class count of {variable_name}:\n')
+        self.logger.info('==========')
+        self.logger.info(f'{class_count}')
+        self.logger.info('==========')
         plt.figure(figsize=(8, 6))
         plt.subplot(1, 1, 1)
         class_count.head(7).plot.bar()
@@ -43,7 +46,7 @@ class TTTH_Analyzer:
         _max = df[variable_name].max()
         _range = _max - _min
         result = {'mean': _mean, 'median': _median, 'mode': _mode[0], 'min': _min, 'max': _max, 'range': _range}
-        print(f'central tendency of {variable_name}: {result}')
+        self.logger.info(f'central tendency of {variable_name}: {result}')
 
     def show_dispersion(self, variable_name, df):
         """
@@ -61,7 +64,7 @@ class TTTH_Analyzer:
         skew = df[variable_name].skew()
         kurtosis = df[variable_name].kurtosis()
         result = {'range': _range, 'q1': q1, 'q3': q3, 'iqr': iqr, 'var': var, 'skew': skew, 'kurtosis': kurtosis}
-        print(f'Dispersion of {variable_name}: \n {result}')
+        self.logger.info(f'Dispersion of {variable_name}: \n {result}')
 
     def visualize_hist_box_plot(self, variable_name, df):
         """
@@ -86,11 +89,11 @@ class TTTH_Analyzer:
         parameter: variable_name: name of continuous variable
         parameter df: DataFrame include continuous column
         """
-        print('=====')
+        self.logger.info('=====')
         self.show_central_tendency(variable_name, df)
-        print('=====')
+        self.logger.info('=====')
         self.show_dispersion(variable_name, df)
-        print('=====')
+        self.logger.info('=====')
         self.visualize_hist_box_plot(variable_name, df)
 
     @staticmethod
@@ -141,7 +144,7 @@ class TTTH_Analyzer:
         parameter df: DataFrame include 2 category variables input
         """
         tw_table = self.create_tw_table(var1, var2, df)
-        print(f'=====Analyze of {var1} and {var2}=====')
+        self.logger.info(f'=====Analyze of {var1} and {var2}=====')
         tw_table.plot(kind='bar', stacked=True, legend=False)
         chi2_result = self.use_chi_2_evaluation(tw_table, prob)
         plt.title(chi2_result)
@@ -162,6 +165,7 @@ class TTTH_Analyzer:
         else:
             raise ValueError('Only support for 2 categories variable analysis')
         plt.legend([], [], frameon=False)
+        plt.xticks(rotation=90)
         plt.show()
 
     def analyze_anova_table_for_continous_vs_categories(self, continous_var, category_vars, df):
@@ -179,7 +183,7 @@ class TTTH_Analyzer:
             raise ValueError('Only support for 2 categories variable analysis')
         model = ols(function, data=df).fit()
         anova_table = sm.stats.anova_lm(model, typ=2)
-        print(anova_table)
+        self.logger.info(anova_table)
 
     def analyze_continous_vs_categories(self, continous_var, category_vars, df):
         """
@@ -189,5 +193,56 @@ class TTTH_Analyzer:
         parameter df: DataFrame contains continuous and category variables input
         """
         self.analyze_anova_table_for_continous_vs_categories(continous_var, category_vars, df)
-        print('======')
+        self.logger.info('======')
         self.visualize_box_for_continous_vs_categories(continous_var, category_vars, df)
+
+    @staticmethod
+    def _count_values_of_variable(variable_name, df, normalize=False):
+        """
+        Count sub categories of category variable
+        :param df: DataFrame
+        :param variable_name: category variable name
+        :param normalize: convert to ratio or not
+        :return:
+        """
+        df_category = df[variable_name]
+        class_count = df_category.value_counts(normalize=normalize)
+        return class_count
+
+    def check_imbalance_class(self, variable_name, df):
+        class_count = self._count_values_of_variable(variable_name, df, normalize=True)
+        self.logger.info(f'Class count of {variable_name}:\n')
+        self.logger.info('==========')
+        self.logger.info(f'{class_count}')
+        self.logger.info('==========')
+        max_class = class_count.max()
+        min_class = class_count.min()
+        ratio_of_classes = max_class/min_class
+        self.logger.info(f'Ratio of 2 class is {ratio_of_classes}')
+        if ratio_of_classes >=2:
+            self.logger.info(f'You should consider to handle imbalance')
+
+    def check_outlier_of_numerical_variable(self, numerical_variable, df):
+        """
+        Function check outlier of variable and return index of outlier if any
+        :param numerical_variable: Name of Numerical variable
+        :param df: DataFrame contains Numerical variable
+        :return: Index of upper and lower outlier
+        """
+        total_sample = df.shape[0]
+        q1 = df[numerical_variable].quantile(0.25)
+        q3 = df[numerical_variable].quantile(0.75)
+        iqr = q3 - q1
+        limit_up = q3 + 1.5 * iqr
+        limit_low = q1 - 1.5 * iqr
+        index_up = df[df[numerical_variable] > limit_up].index
+        index_low = df[df[numerical_variable] < limit_low].index
+        upper_outlier_ratio = index_up.shape[0]/total_sample
+        lower_outlier_ratio = index_low.shape[0]/total_sample
+        if index_up.empty and index_low.empty:
+            self.logger.info(f'Variable {numerical_variable} have no outlier')
+            return
+        else:
+            self.logger.info(f'variable {numerical_variable} have {round(upper_outlier_ratio)}% upper outlier')
+            self.logger.info(f'variable {numerical_variable} have {round(lower_outlier_ratio)}% lower outlier')
+            return index_up, index_low
